require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'
require 'csv'

module StudioGame
  class Game

    attr_reader :title

    def initialize(title)
      @title = title
      @players = []
    end
  
    def print_name_and_health(player)
      puts "#{player.name} (#{player.health})"
    end
  
    def print_stats
      puts "\n#{@title} Statistics:\n"
      # strong_players, wimpy_players = @players.partition { |player| player.strong? }
      # 
      # puts "\n#{strong_players.size} strong players:"
      # strong_players.each do |player|
      #   print_name_and_health(player)
      # end
      # 
      # puts "\n#{wimpy_players.size} wimpy players:"
      # wimpy_players.each do |player|
      #   print_name_and_health(player)
      # end
    
      @players.each do |player|
        puts "#{player.name}'s point totals:"
      
        # B
        player.each_found_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points"
        end
      
        # another way (alternative for player.each_found_treasure above)
        player.practice
      
        puts "#{player.points} GRAND TOTAL points\n\n"
      end
    
    
      puts "\n#{title} High Scores:"
      @players.sort.each do |player|
        puts print_high_scores(player)
      end
    
    
      puts "Total found treasures by all players: #{total_points}"
      puts "\n----------------------------------------------------------"

    end
  
  
    def total_points
      @players.reduce(0){|sum, player| sum + player.points}
    end
     
  
    def add_player(a_player)
      @players << a_player
    end
  
    def play(rounds)

      puts "There are #{@players.size} players in #{@title}:\n\n"
     
      @players.each do |player|
        puts player
      end
    
      treasures = TreasureTrove::TREASURES
    
      puts "There are #{treasures.size} treasures to be found:"
      treasures.each do |treasure|
        puts "A #{treasure.name} is worth #{treasure.points} points"
      end
    
      puts "\n*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *"
    
      1.upto(rounds) do |round|
      
        if block_given?
          break if yield
        end
      
        puts "\nRound #{round}:\n\n"
        @players.each do |player|
          GameTurn.take_turn(player)  
          puts player
        
        end
        print_stats
      end
    end
  
    # Input / Output
  
    def print_high_scores(player)
      formatted_name = player.name.ljust(20, '.')
      "#{formatted_name} #{player.score}"
    end
  
  
    # def turning_CSV_string_into_a_Player_object(line)
    #   name, health = line.split(',')
    #   player = Player.new(name, Integer(health))
    # end
  
    def load_players(from_file)
      CSV.foreach(from_file) do |row|
        player = Player.new(row[0], row[1].to_i)
        add_player(player)
    
      # File.readlines(from_file).each do |line|
      # add_player(turning_CSV_string_into_a_Player_object(line))
      
      end
    end
  
    def save_high_scores(to_file="high_scores.txt")
      File.open(to_file, "w") do |file|
        file.puts "#{@title} High Scores:"
        @players.sort.each do |player|
          file.puts print_high_scores(player)
        end
      end
    end
  end
end


if __FILE__ == $0
player1 = Player.new("moe")
player2 = Player.new("larry", 60)
knuckleheads = Game.new("Knuckleheads")
puts knuckleheads.title
knuckleheads.add_player(player1)
knuckleheads.add_player(player2)
#knuckleheads.print_name_and_health(player1)

knuckleheads.play(1)
end